
#line 1 "../../../src/main/http_range_parser.rl"

#include <lighttpd/base.h>
#include <lighttpd/http_range_parser.h>

#include <stdlib.h>


#line 11 "http_range_parser.c"
static const char _http_range_parser_key_offsets[] = {
	0, 0, 3, 4, 5, 6, 7, 10, 
	16, 20, 25, 28, 33, 36, 41, 44, 
	50, 55, 60
};

static const char _http_range_parser_trans_keys[] = {
	9, 32, 98, 121, 116, 101, 115, 9, 
	32, 61, 9, 32, 44, 45, 48, 57, 
	9, 32, 48, 57, 9, 32, 45, 48, 
	57, 9, 32, 45, 9, 32, 45, 48, 
	57, 9, 32, 45, 9, 32, 44, 48, 
	57, 9, 32, 44, 9, 32, 44, 45, 
	48, 57, 9, 32, 44, 48, 57, 9, 
	32, 44, 48, 57, 9, 32, 44, 48, 
	57, 0
};

static const char _http_range_parser_single_lengths[] = {
	0, 3, 1, 1, 1, 1, 3, 4, 
	2, 3, 3, 3, 3, 3, 3, 4, 
	3, 3, 3
};

static const char _http_range_parser_range_lengths[] = {
	0, 0, 0, 0, 0, 0, 0, 1, 
	1, 1, 0, 1, 0, 1, 0, 1, 
	1, 1, 1
};

static const char _http_range_parser_index_offsets[] = {
	0, 0, 4, 6, 8, 10, 12, 16, 
	22, 26, 31, 35, 40, 44, 49, 53, 
	59, 64, 69
};

static const char _http_range_parser_indicies[] = {
	0, 0, 2, 1, 3, 1, 4, 1, 
	5, 1, 6, 1, 6, 6, 7, 1, 
	7, 7, 7, 8, 9, 1, 8, 8, 
	10, 1, 11, 11, 12, 13, 1, 14, 
	14, 15, 1, 16, 16, 17, 18, 1, 
	19, 19, 20, 1, 21, 21, 22, 23, 
	1, 24, 24, 25, 1, 26, 26, 26, 
	8, 27, 1, 15, 15, 28, 29, 1, 
	30, 30, 31, 32, 1, 33, 33, 28, 
	29, 1, 0
};

static const char _http_range_parser_trans_targs[] = {
	1, 0, 2, 3, 4, 5, 6, 7, 
	8, 11, 13, 10, 16, 9, 10, 16, 
	12, 18, 11, 12, 18, 14, 15, 13, 
	14, 15, 15, 9, 15, 17, 14, 15, 
	17, 18
};

static const char _http_range_parser_trans_actions[] = {
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 1, 1, 2, 2, 3, 0, 0, 
	2, 2, 3, 0, 0, 4, 5, 3, 
	0, 6, 0, 1, 8, 1, 9, 10, 
	3, 7
};

static const char _http_range_parser_eof_actions[] = {
	0, 0, 0, 0, 0, 0, 0, 0, 
	0, 0, 0, 0, 0, 4, 0, 0, 
	7, 9, 7
};

static const int http_range_parser_start = 1;
static const int http_range_parser_first_final = 13;
static const int http_range_parser_error = 0;

static const int http_range_parser_en_main = 1;


#line 56 "../../../src/main/http_range_parser.rl"


liParseHttpRangeResult li_parse_http_range_next(liParseHttpRangeState* s) {
	const char *pe, *eof;
	goffset tmp = 0;

	if (s->cs == http_range_parser_error) {
		return LI_PARSE_HTTP_RANGE_INVALID;
	}

	eof = pe = s->data->str + s->data->len;

	for ( ;; ) {
		gboolean found = FALSE;

		if (s->data_pos >= eof) {
			return s->found_valid_range ? LI_PARSE_HTTP_RANGE_DONE : LI_PARSE_HTTP_RANGE_NOT_SATISFIABLE;
		}

		
#line 111 "http_range_parser.c"
	{
	int _klen;
	const char *_keys;
	int _trans;

	if ( ( s->data_pos) == pe )
		goto _test_eof;
	if ( ( s->cs) == 0 )
		goto _out;
_resume:
	_keys = _http_range_parser_trans_keys + _http_range_parser_key_offsets[( s->cs)];
	_trans = _http_range_parser_index_offsets[( s->cs)];

	_klen = _http_range_parser_single_lengths[( s->cs)];
	if ( _klen > 0 ) {
		const char *_lower = _keys;
		const char *_mid;
		const char *_upper = _keys + _klen - 1;
		while (1) {
			if ( _upper < _lower )
				break;

			_mid = _lower + ((_upper-_lower) >> 1);
			if ( (*( s->data_pos)) < *_mid )
				_upper = _mid - 1;
			else if ( (*( s->data_pos)) > *_mid )
				_lower = _mid + 1;
			else {
				_trans += (_mid - _keys);
				goto _match;
			}
		}
		_keys += _klen;
		_trans += _klen;
	}

	_klen = _http_range_parser_range_lengths[( s->cs)];
	if ( _klen > 0 ) {
		const char *_lower = _keys;
		const char *_mid;
		const char *_upper = _keys + (_klen<<1) - 2;
		while (1) {
			if ( _upper < _lower )
				break;

			_mid = _lower + (((_upper-_lower) >> 1) & ~1);
			if ( (*( s->data_pos)) < _mid[0] )
				_upper = _mid - 2;
			else if ( (*( s->data_pos)) > _mid[1] )
				_lower = _mid + 2;
			else {
				_trans += ((_mid - _keys)>>1);
				goto _match;
			}
		}
		_trans += _klen;
	}

_match:
	_trans = _http_range_parser_indicies[_trans];
	( s->cs) = _http_range_parser_trans_targs[_trans];

	if ( _http_range_parser_trans_actions[_trans] == 0 )
		goto _again;

	switch ( _http_range_parser_trans_actions[_trans] ) {
	case 3:
#line 20 "../../../src/main/http_range_parser.rl"
	{
		int d = (*( s->data_pos)) - '0';
		if (tmp > (G_MAXOFFSET-10)/10) {
			s->cs = http_range_parser_error;
			return LI_PARSE_HTTP_RANGE_INVALID;
		}
		tmp = 10*tmp + d;
	}
	break;
	case 2:
#line 31 "../../../src/main/http_range_parser.rl"
	{
		s->range_start = tmp;
	}
	break;
	case 9:
#line 34 "../../../src/main/http_range_parser.rl"
	{
		s->range_end = tmp;
		found = TRUE;
	}
	break;
	case 7:
#line 38 "../../../src/main/http_range_parser.rl"
	{
		s->range_end = s->limit - 1;
		found = TRUE;
	}
	break;
	case 4:
#line 42 "../../../src/main/http_range_parser.rl"
	{
		s->range_end = s->limit - 1;
		s->range_start = s->limit - tmp;
		found = TRUE;
	}
	break;
	case 6:
#line 47 "../../../src/main/http_range_parser.rl"
	{
		{( s->data_pos)++; goto _out; }
	}
	break;
	case 1:
#line 17 "../../../src/main/http_range_parser.rl"
	{
		tmp = 0;
	}
#line 20 "../../../src/main/http_range_parser.rl"
	{
		int d = (*( s->data_pos)) - '0';
		if (tmp > (G_MAXOFFSET-10)/10) {
			s->cs = http_range_parser_error;
			return LI_PARSE_HTTP_RANGE_INVALID;
		}
		tmp = 10*tmp + d;
	}
	break;
	case 10:
#line 34 "../../../src/main/http_range_parser.rl"
	{
		s->range_end = tmp;
		found = TRUE;
	}
#line 47 "../../../src/main/http_range_parser.rl"
	{
		{( s->data_pos)++; goto _out; }
	}
	break;
	case 8:
#line 38 "../../../src/main/http_range_parser.rl"
	{
		s->range_end = s->limit - 1;
		found = TRUE;
	}
#line 47 "../../../src/main/http_range_parser.rl"
	{
		{( s->data_pos)++; goto _out; }
	}
	break;
	case 5:
#line 42 "../../../src/main/http_range_parser.rl"
	{
		s->range_end = s->limit - 1;
		s->range_start = s->limit - tmp;
		found = TRUE;
	}
#line 47 "../../../src/main/http_range_parser.rl"
	{
		{( s->data_pos)++; goto _out; }
	}
	break;
#line 272 "http_range_parser.c"
	}

_again:
	if ( ( s->cs) == 0 )
		goto _out;
	if ( ++( s->data_pos) != pe )
		goto _resume;
	_test_eof: {}
	if ( ( s->data_pos) == eof )
	{
	switch ( _http_range_parser_eof_actions[( s->cs)] ) {
	case 9:
#line 34 "../../../src/main/http_range_parser.rl"
	{
		s->range_end = tmp;
		found = TRUE;
	}
	break;
	case 7:
#line 38 "../../../src/main/http_range_parser.rl"
	{
		s->range_end = s->limit - 1;
		found = TRUE;
	}
	break;
	case 4:
#line 42 "../../../src/main/http_range_parser.rl"
	{
		s->range_end = s->limit - 1;
		s->range_start = s->limit - tmp;
		found = TRUE;
	}
	break;
#line 306 "http_range_parser.c"
	}
	}

	_out: {}
	}

#line 76 "../../../src/main/http_range_parser.rl"

		if (s->cs == http_range_parser_error) {
			return LI_PARSE_HTTP_RANGE_INVALID;
		}

		if (s->data_pos >= eof) {
			s->last_range = TRUE;
		}

		if (!found) {
			return s->found_valid_range ? LI_PARSE_HTTP_RANGE_DONE : LI_PARSE_HTTP_RANGE_NOT_SATISFIABLE;
		}

		if (s->range_end >= s->limit) {
			s->range_end = s->limit - 1;
		}
		if (s->range_start < 0) {
			s->range_start = 0;
		}

		if (s->range_start <= s->range_end) {
			s->found_valid_range = TRUE;
			s->range_length = s->range_end - s->range_start + 1;
			return LI_PARSE_HTTP_RANGE_OK;
		}
	}
}

void li_parse_http_range_init(liParseHttpRangeState* s, const GString *range_str, goffset limit) {
	s->data = g_string_new_len(GSTR_LEN(range_str));
	s->data_pos = s->data->str;
	s->limit = limit;
	s->last_range = FALSE;
	s->found_valid_range = FALSE;

	
#line 350 "http_range_parser.c"
	{
	( s->cs) = http_range_parser_start;
	}

#line 112 "../../../src/main/http_range_parser.rl"
}

void li_parse_http_range_clear(liParseHttpRangeState* s) {
	if (s->data) {
		g_string_free(s->data, TRUE);
		s->data = NULL;
	}
}
